"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveExpression = exports.isExpression = void 0;
const message_value_1 = require("../message-value");
const message_value_2 = require("../message-value/message-value");
const literal_1 = require("./literal");
/**
 * Type guard for {@link Expression} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isExpression = (part) => !!part && typeof part === 'object' && part.type === 'expression';
exports.isExpression = isExpression;
function resolveOptions(ctx, options, expected) {
    const opt = { localeMatcher: ctx.localeMatcher };
    const errorKeys = [];
    if (options && expected) {
        for (const { name, value } of options) {
            const exp = typeof expected === 'string' || Array.isArray(expected)
                ? expected
                : expected[name];
            const { res, valid } = resolveOptionValue(ctx, exp, value);
            opt[name] = res;
            if (!valid)
                errorKeys.push(name);
        }
    }
    return { opt, errorKeys };
}
function resolveOptionValue(ctx, exp, elem) {
    const { value } = ctx.resolve(elem);
    if (exp === 'any' ||
        exp === typeof value ||
        (Array.isArray(exp) && typeof value === 'string' && exp.includes(value))) {
        return { res: value, valid: true };
    }
    if ((0, literal_1.isLiteral)(elem)) {
        switch (exp) {
            case 'boolean':
                if (value === 'true')
                    return { res: true, valid: true };
                if (value === 'false')
                    return { res: false, valid: true };
                return { res: value === '0' ? false : Boolean(value), valid: false };
            case 'number': {
                const num = Number(value);
                return { res: num, valid: Number.isFinite(num) };
            }
        }
    }
    return { res: value, valid: false };
}
function resolveExpression(ctx, { operand, name, options }) {
    let source;
    const rf = ctx.runtime[name];
    try {
        let fnArgs;
        if (operand) {
            const arg = ctx.resolve(operand);
            fnArgs = [arg];
            const argSrc = arg.source || (arg.type === 'literal' && arg.value) || message_value_2.FALLBACK_SOURCE;
            source = `${argSrc} :${name}`;
        }
        else {
            fnArgs = [];
            source = `:${name}`;
        }
        if (!rf) {
            const error = new ReferenceError(`Unknown formatter ${name}`);
            throw Object.assign(error, { type: 'missing-func' });
        }
        const { opt, errorKeys } = resolveOptions(ctx, options, rf.options);
        const res = rf.call(ctx.locales, opt, ...fnArgs);
        const mv = (0, message_value_1.asMessageValue)(ctx, res, { source });
        for (const key of errorKeys) {
            const error = new TypeError(`Invalid value for option ${key}`);
            ctx.onError(Object.assign(error, { type: 'invalid-type' }), mv);
        }
        return mv;
    }
    catch (error) {
        source !== null && source !== void 0 ? source : (source = `${name}(${operand ? message_value_2.FALLBACK_SOURCE : ''})`);
        const fb = new message_value_1.MessageFallback(ctx, { source });
        ctx.onError(error, fb);
        return fb;
    }
}
exports.resolveExpression = resolveExpression;
